package com.shcd.service.impl;

import cn.hutool.core.lang.UUID;
import cn.hutool.crypto.asymmetric.RSA;
import com.shcd.constant.CommonKeys;
import com.shcd.entity.LicenceDTO;
import com.shcd.service.LicenceService;
import com.shcd.util.LicenceJsonUtil;
import com.shcd.util.NativeSecurityUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

/**
 * @author 上海城地
 * @description 证书生成核心实现类
 * @create 2024-04-13 15:55
 * @since 1.0.0
 */
@Service
public class LicenceServiceImpl implements LicenceService {

    private static Logger log = LoggerFactory.getLogger(LicenceServiceImpl.class);


    /**
     * 创建证书 licence 内容
     *
     * @param dtoEntity
     * @return
     */
    @Override
    public LicenceDTO createLicence(LicenceDTO dtoEntity) {
        RSA rsa = new RSA();
        LicenceDTO entity = new LicenceDTO();
        //赋值相同属性
        BeanUtils.copyProperties(dtoEntity, entity);
        entity.setLicenceId(UUID.randomUUID().toString());
        //content 和 key 需要额外处理
        entity.setKey(new String(rsa.getPublicKey().getEncoded()));//把公钥放进去，否则客户端无法获取公钥，就无法解密
        //把实体转成字符串
        String json = LicenceJsonUtil.objectToStr(entity);
        //把整个字符串加密
        String content = NativeSecurityUtil.encryptByPrivateKey(json, new String(rsa.getPrivateKey().getEncoded()));
        //把加密后的字符串赋值给 content
        entity.setContent(content);
        return entity;
    }

//    /**
//     * 下载证书文件
//     *
//     * @param dtoEntity
//     * @param response
//     */
//    @Override
//    public void downLoadLicence(LicenceDTO dtoEntity, HttpServletResponse response) {
//        LicenceDTO licenceEntity = createLicence(dtoEntity);
//        if (null == licenceEntity) {
//            log.error("证书的秘钥未配置！");
//            return;
//        }
//        //把实体转为字符串
//        String result = LicenceJsonUtil.objectToStr(licenceEntity);
//        BufferedOutputStream out = null;
//        try {
//            //证书文件名
//            String fileName = CommonKeys.CERTIFICATE_FILE;
//            response.setContentType("application/octet-stream");
//            response.setHeader("Content-Disposition", "attachment; filename=\"" + fileName + "\"");
//            response.setCharacterEncoding("UTF-8");
//            out = new BufferedOutputStream(response.getOutputStream());
//            out.write(result.getBytes("UTF-8"));
//            out.flush();
//        } catch (IOException e) {
//            log.error(e.getMessage(), e);
//        } finally {
//            try {
//                if (out != null) {
//                    out.close();
//                }
//            } catch (Exception e) {
//                log.error(e.getMessage(), e);
//            }
//        }
//    }

    /**
     * 本地生成证书
     *
     * @param dtoEntity
     * @return boolean
     * @author Hu
     * @date 2024/11/20 17:13
     */
    @Override
    public void locallyGenerated(LicenceDTO dtoEntity) {
        // 生成证书实体类
        LicenceDTO licenceEntity = createLicence(dtoEntity);
        //把实体转为字符串
        String result = LicenceJsonUtil.objectToStr(licenceEntity);
        saveLicenceToCertificate(dtoEntity, result);
    }

    /**
     * 保存证书文件到指定目录
     *
     * @param result
     */
    public void saveLicenceToCertificate(LicenceDTO dtoEntity, String result) {
        try {
            // 创建证书目录（如果尚未创建）
            Path directory = Paths.get(null == dtoEntity.getCertificateDirectory() ? CommonKeys.CERTIFICATE_DIRECTORY : dtoEntity.getCertificateDirectory());
            Files.createDirectories(directory);
            // 构建证书文件的完整路径
            Path filePath = directory.resolve(null == dtoEntity.getCertificateFile() ? CommonKeys.CERTIFICATE_FILE : dtoEntity.getCertificateFile());
            // 检查文件是否存在，存在则删掉
            Files.deleteIfExists(filePath);
            //创建文件（如果文件已经存在，此步骤可能会抛出 FileAlreadyExistsException）
            Files.createFile(filePath);
            //写入内容到文件，使用 StandardOpenOption.APPEND 可以追加内容而不是覆盖
            Files.write(filePath, result.getBytes("UTF-8"), StandardOpenOption.CREATE, StandardOpenOption.WRITE, StandardOpenOption.TRUNCATE_EXISTING);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

}
